#!/usr/bin/env python

# Data from OpenWeatherMap
# show_graph function adapted from cpu_temp.py

from sys import exit
from time import sleep

try:
    import requests
except ImportError:
    exit('This script requires the requests module\nInstall with: sudo pip install requests')

import ledshim

# Grab your API key here: http://openweathermap.org
# List of city ID city.list.json.gz can be downloaded here http://bulk.openweathermap.org/sample/
API_KEY = ''
CITY_ID = ''

url = 'http://api.openweathermap.org/data/2.5/weather'

temp = 0


def update_weather():
    payload = {
        'id': CITY_ID,
        'units': 'metric',
        'appid': API_KEY
    }
    global temp
    try:
        r = requests.get(url=url, params=payload)
        temp = r.json().get('main').get('temp')
        print('Temperture = ' + str(temp) + ' C')
    except (requests.ConnectionError, requests.ConnectTimeout):
        print('Connection Error')


def show_graph(v, r, g, b):
    v *= ledshim.NUM_PIXELS
    for x in range(ledshim.NUM_PIXELS):
        if v < 0:
            r, g, b = 0, 0, 0
        else:
            r, g, b = [int(min(v, 1.0) * c) for c in [r, g, b]]
        ledshim.set_pixel(x, r, g, b)
        v -= 1
    ledshim.show()


def draw_thermo(temp):
    v = temp
    v /= 40
    v += (1 / 8)
    show_graph(v, 255, 0, 0)


ledshim.set_brightness(0.1)

while 1:
    update_weather()
    draw_thermo(temp)
    sleep(120)



#!/usr/bin/env python

import colorsys
import time

import ledshim

spacing = 360.0 / 16.0
hue = 0

ledshim.set_clear_on_exit()
ledshim.set_brightness(0.8)

while True:
    hue = int(time.time() * 100) % 360
    for x in range(ledshim.NUM_PIXELS):
        offset = x * spacing
        h = ((hue + offset) % 360) / 360.0
        r, g, b = [int(c * 255) for c in colorsys.hsv_to_rgb(h, 1.0, 1.0)]
        ledshim.set_pixel(x, r, g, b)

    ledshim.show()
    time.sleep(0.0001)



#!/usr/bin/env python

import time
from sys import exit

try:
    from tweepy import OAuthHandler, Stream
    from tweepy.streaming import StreamListener
except ImportError:
    exit('This script requires the tweepy module\nInstall with: sudo pip install tweepy')

import ledshim

ckey = ''     # Consumer key
csecret = ''  # Consumer secret
atoken = ''   # Access token
asecret = ''  # Access secret


class listener(StreamListener):
    def on_data(self, data):
        blink_ledshim()
        return True

    def on_error(self, status):
        print(status)


def blink_ledshim():
    for i in range(3):
        for j in range(ledshim.NUM_PIXELS):
            ledshim.set_pixel(j, 255, 0, 0)

        ledshim.show()
        time.sleep(0.1)

        for j in range(ledshim.NUM_PIXELS):
            ledshim.set_pixel(j, 0, 0, 0)

        ledshim.show()
        time.sleep(0.2)


auth = OAuthHandler(ckey, csecret)
auth.set_access_token(atoken, asecret)
twitterstream = Stream(auth, listener())
twitterstream.filter(track=['#INSERTHASHTAGHERE'])



#!/usr/bin/env python

import time

import ledshim

r, g, b = 255, 0, 0

delay = 0.1

half_way = ledshim.NUM_PIXELS // 2

while True:
    # Turn pixels on
    for x in range(half_way):
        ledshim.set_pixel(half_way - 1 - x, r, g, b)
        ledshim.set_pixel(half_way + x, r, g, b)
        ledshim.show()
        time.sleep(delay)

    # Turn pixels off
    for x in range(half_way):
        ledshim.set_pixel(x, 0, 0, 0)
        ledshim.set_pixel(ledshim.NUM_PIXELS - 1 - x, 0, 0, 0)
        ledshim.show()
        time.sleep(delay)



#!/usr/bin/env python

import time

import ledshim

ledshim.set_clear_on_exit()

REDS = [0] * ledshim.NUM_PIXELS * 2
SCAN = [1, 2, 4, 8, 16, 32, 64, 128, 255]
REDS[ledshim.NUM_PIXELS - len(SCAN):ledshim.NUM_PIXELS + len(SCAN)] = SCAN + SCAN[::-1]

start_time = time.time()

while True:
    # Sine wave, spends a little longer at min/max
    # delta = (time.time() - start_time) * 8
    # offset = int(round(((math.sin(delta) + 1) / 2) * (ledshim.NUM_PIXELS - 1)))

    # Triangle wave, a snappy ping-pong effect
    delta = (time.time() - start_time) * ledshim.NUM_PIXELS * 2
    offset = int(abs((delta % len(REDS)) - ledshim.NUM_PIXELS))

    for i in range(ledshim.NUM_PIXELS):
        ledshim.set_pixel(i, REDS[offset + i], 0, 0)

    ledshim.show()

    time.sleep(0.05)



#!/usr/bin/env python

import random
import time

import ledshim

ledshim.set_clear_on_exit()

while True:
    pixels = random.sample(range(ledshim.NUM_PIXELS), random.randint(1, 5))
    for i in range(ledshim.NUM_PIXELS):
        if i in pixels:
            ledshim.set_pixel(i, 255, 150, 0)
        else:
            ledshim.set_pixel(i, 0, 0, 0)
    ledshim.show()
    time.sleep(0.05)



#!/usr/bin/env python
import time

import ledshim

for col in ((255, 0, 0), (0, 255, 0), (0, 0, 255)):
    r, g, b = col
    for x in range(ledshim.DISPLAY_WIDTH):
        ledshim.clear()
        ledshim.set_pixel(x, r, g, b)
        ledshim.show()
        time.sleep(0.2)



"""Driver for the IS31FL3731."""

import atexit
import time

import smbus2

_MODE_REGISTER = 0x00
_FRAME_REGISTER = 0x01
_AUTOPLAY1_REGISTER = 0x02
_AUTOPLAY2_REGISTER = 0x03
_BLINK_REGISTER = 0x05
_AUDIOSYNC_REGISTER = 0x06
_BREATH1_REGISTER = 0x08
_BREATH2_REGISTER = 0x09
_SHUTDOWN_REGISTER = 0x0A
_GAIN_REGISTER = 0x0B
_ADC_REGISTER = 0x0C

_CONFIG_BANK = 0x0B
_BANK_ADDRESS = 0xFD

_PICTURE_MODE = 0x00
_AUTOPLAY_MODE = 0x08
_AUDIOPLAY_MODE = 0x18

_ENABLE_OFFSET = 0x00
_BLINK_OFFSET = 0x12
_COLOR_OFFSET = 0x24

LED_GAMMA = [
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2,
    2, 2, 2, 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5,
    6, 6, 6, 7, 7, 7, 8, 8, 8, 9, 9, 9, 10, 10, 11, 11,
    11, 12, 12, 13, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17, 18, 18,
    19, 19, 20, 21, 21, 22, 22, 23, 23, 24, 25, 25, 26, 27, 27, 28,
    29, 29, 30, 31, 31, 32, 33, 34, 34, 35, 36, 37, 37, 38, 39, 40,
    40, 41, 42, 43, 44, 45, 46, 46, 47, 48, 49, 50, 51, 52, 53, 54,
    55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70,
    71, 72, 73, 74, 76, 77, 78, 79, 80, 81, 83, 84, 85, 86, 88, 89,
    90, 91, 93, 94, 95, 96, 98, 99, 100, 102, 103, 104, 106, 107, 109, 110,
    111, 113, 114, 116, 117, 119, 120, 121, 123, 124, 126, 128, 129, 131, 132, 134,
    135, 137, 138, 140, 142, 143, 145, 146, 148, 150, 151, 153, 155, 157, 158, 160,
    162, 163, 165, 167, 169, 170, 172, 174, 176, 178, 179, 181, 183, 185, 187, 189,
    191, 193, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220,
    222, 224, 227, 229, 231, 233, 235, 237, 239, 241, 244, 246, 248, 250, 252, 255]


class Matrix:
    """Represent an IS31LF3731 Matrix Display."""

    _width = 28
    _height = 1

    def __init__(self, i2c, address=0x74, gamma_table=None):
        """Initialise Matrix.

        :param i2c: SMBus-compatible i2s bus device
        :param address: i2c address
        :param gamma_table: list of 256 gamma correction values

        """
        self.i2c = i2c
        self.address = address
        self._is_setup = False
        self._clear_on_exit = True

        if gamma_table is None:
            gamma_table = list(range(256))

        self._gamma_table = gamma_table

        self._brightness = 1.0

        self.clear()

    def setup(self):
        """Set up device."""
        if self._is_setup:
            return True

        self._is_setup = True

        if self.i2c is None:
            try:
                self.i2c = smbus2.SMBus(1)
            except IOError as e:
                if hasattr(e, "errno") and e.errno == 2:
                    e.strerror += "\n\nMake sure you've enabled i2c in your Raspberry Pi configuration.\n"
                raise e

        try:
            self._reset()
        except IOError as e:
            if hasattr(e, "errno") and e.errno == 5:
                e.strerror += "\n\nMake sure your LED SHIM is attached, and double-check your soldering.\n"
            raise e

        self.show()

        # Display initialization

        # Switch to configuration bank
        self._bank(_CONFIG_BANK)

        # Switch to Picture Mode
        self.i2c.write_i2c_block_data(self.address, _MODE_REGISTER, [_PICTURE_MODE])

        # Disable audio sync
        self.i2c.write_i2c_block_data(self.address, _AUDIOSYNC_REGISTER, [0])

        enable_pattern = [
            0b00000000, 0b10111111,
            0b00111110, 0b00111110,
            0b00111111, 0b10111110,
            0b00000111, 0b10000110,
            0b00110000, 0b00110000,
            0b00111111, 0b10111110,
            0b00111111, 0b10111110,
            0b01111111, 0b11111110,
            0b01111111, 0b00000000,
        ]

        # Switch to bank 1 ( frame 1 )
        self._bank(1)

        # Enable LEDs
        self.i2c.write_i2c_block_data(self.address, 0x00, enable_pattern)

        # Switch to bank 0 ( frame 0 )
        self._bank(0)

        # Enable LEDs
        self.i2c.write_i2c_block_data(self.address, 0x00, enable_pattern)

        atexit.register(self._exit)

    def set_clear_on_exit(self, value=True):
        """Set whether LED SHIM should be cleared upon exit.

        By default LED SHIM will turn off the pixels on exit, but calling::

            scrollphathd.set_clear_on_exit(False)

        Will ensure that it does not.

        :param value: True or False (default True)

        """
        self._clear_on_exit = value

    def _exit(self):
        if self._clear_on_exit:
            self.clear()
            self.show()

    @property
    def width(self):
        """Return width of device."""
        return self._width

    @property
    def height(self):
        """Return height of device."""
        return self._height

    def set_gamma(self, gamma_table):
        """Set the LED gamma table.

        Set the table of values used to give the LEDs a pleasing
        to the eye brightness curve.

        :param gamma_table: List of 256 values in the range 0-255.

        """
        if len(gamma_table) != 256:
            raise ValueError("Gamma table must be a list with 256 values.")

        self._gamma_table = gamma_table

    def clear(self):
        """Clear the buffer.

        You must call `show` after clearing the buffer to update the display.

        """
        self._current_frame = 0

        try:
            del self.buf
        except AttributeError:
            pass

        self.buf = [[0, 0, 0, 1.0] for x in range(self._width)]

    def set_brightness(self, brightness):
        """Set a global brightness value.

        :param brightness: Brightness value from 0.0 to 1.0

        """
        self._brightness = brightness

    def set_all(self, r, g, b, brightness=1.0):
        """Set all pixels in the buffer.

        :param r, g, b: Intensity of the pixel, from 0 to 255.

        """
        for x in range(self._width):
            self.set_pixel(x, r, g, b, brightness)

    def set_pixel(self, x, r, g, b, brightness=1.0):
        """Set a single pixel in the buffer.

        :param x: Position of pixel from left
        :param r, g, b: Intensity of the pixel, from 0 to 255.

        """
        r, g, b = [int(c) for c in (r, g, b)]

        for c in (r, g, b):
            if c > 255 or c < 0:
                raise ValueError(f"Value {c} out of range. RGB values should be between 0 and 255")

        try:
            self.buf[x] = r, g, b, brightness

        except IndexError:
            raise ValueError(f"x position ({x}) is out of range!")

    def get_shape(self):
        """Get the size/shape of the display.

        Returns a tuple containing the width and height of the display,
        after applying rotation.

        """
        return (self._width, self._height)

    def show(self):
        """Show the buffer contents on the display."""
        self.setup()

        next_frame = 0 if self._current_frame == 1 else 0

        output = [0 for x in range(144)]

        for x in range(self._width):
                r, g, b, br = self.buf[x]
                r, g, b = [self._gamma_table[int(c * self._brightness * br)] for c in (r, g, b)]

                rgb = [r, g, b]
                for y in range(3):
                    idx = self._pixel_addr(x, y)
                    output[idx] = rgb[y]

        self._bank(next_frame)

        offset = 0

        for chunk in self._chunk(output, 32):
            self.i2c.write_i2c_block_data(self.address, _COLOR_OFFSET + offset, chunk)
            offset += 32

        self._frame(next_frame)

    def _reset(self):
        self._sleep(True)
        time.sleep(0.00001)
        self._sleep(False)

    def _sleep(self, value):
        return self._register(_CONFIG_BANK, _SHUTDOWN_REGISTER, not value)

    def _frame(self, frame=None, show=True):
        if frame is None:
            return self._current_frame

        if not 0 <= frame <= 8:
            raise ValueError("Frame out of range: 0-8")

        self._current_frame = frame

        if show:
            self._register(_CONFIG_BANK, _FRAME_REGISTER, frame)

    def _bank(self, bank=None):
        """Switch display driver memory bank."""
        if bank is None:
            return self.i2c.readfrom_mem(self.address, _BANK_ADDRESS, 1)[0]

        self.i2c.write_i2c_block_data(self.address, _BANK_ADDRESS, [bank])

    def _register(self, bank, register, value=None):
        """Write display driver register."""
        self._bank(bank)

        if value is None:
            return self.i2c.readfrom_mem(self.address, register, 1)[0]

        self.i2c.write_i2c_block_data(self.address, register, [value])

    def _chunk(self, data, length):
        for i in range(0, len(data) + 1, length):
            yield data[i : i + length]

    def _pixel_addr(self, x, y):
        return x + y * 16


class LEDSHIM(Matrix):
    """LED SHIM."""

    width = 28
    height = 1

    def _pixel_addr(self, x, rgb):
        lookup = [
            (118, 69, 85),
            (117, 68, 101),
            (116, 84, 100),
            (115, 83, 99),
            (114, 82, 98),
            (113, 81, 97),
            (112, 80, 96),
            (134, 21, 37),
            (133, 20, 36),
            (132, 19, 35),
            (131, 18, 34),
            (130, 17, 50),
            (129, 33, 49),
            (128, 32, 48),

            (127, 47, 63),
            (121, 41, 57),
            (122, 25, 58),
            (123, 26, 42),
            (124, 27, 43),
            (125, 28, 44),
            (126, 29, 45),
            (15, 95, 111),
            (8, 89, 105),
            (9, 90, 106),
            (10, 91, 107),
            (11, 92, 108),
            (12, 76, 109),
            (13, 77, 93),
        ]
        return lookup[x][rgb]



#!/usr/bin/env python

import colorsys
import time
from sys import exit

try:
    import numpy as np
except ImportError:
    exit('This script requires the numpy module\nInstall with: sudo pip install numpy')

import ledshim

ledshim.set_clear_on_exit()


def make_gaussian(fwhm):
    x = np.arange(0, ledshim.NUM_PIXELS, 1, float)
    y = x[:, np.newaxis]
    x0, y0 = 3.5, (ledshim.NUM_PIXELS / 2) - 0.5
    fwhm = fwhm
    gauss = np.exp(-4 * np.log(2) * ((x - x0) ** 2 + (y - y0) ** 2) / fwhm ** 2)
    return gauss


while True:
    for z in list(range(1, 10)[::-1]) + list(range(1, 10)):
        fwhm = 15.0 / z
        gauss = make_gaussian(fwhm)
        start = time.time()
        y = 4
        for x in range(ledshim.NUM_PIXELS):
            h = 0.5
            s = 1.0
            v = gauss[x, y]
            rgb = colorsys.hsv_to_rgb(h, s, v)
            r, g, b = [int(255.0 * i) for i in rgb]
            ledshim.set_pixel(x, r, g, b)
        ledshim.show()
        end = time.time()
        t = end - start
        if t < 0.04:
            time.sleep(0.04 - t)



"""LED SHIM 28-pixel RGB LED display."""
from . import is31fl3731

__version__ = "0.0.3"

display = is31fl3731.LEDSHIM(None, address=0x75, gamma_table=is31fl3731.LED_GAMMA)

DISPLAY_HEIGHT = display._height
DISPLAY_WIDTH = display._width
NUM_PIXELS = display._width

pixel = display.set_pixel
set_pixel = display.set_pixel
set_all = display.set_all
set_brightness = display.set_brightness
show = display.show
width = display.width
height = display.height
clear = display.clear
get_buffer_shape = display.get_shape
get_shape = display.get_shape
set_clear_on_exit = display.set_clear_on_exit


def set_multiple_pixels(indexes, from_colour, to_colour=None):
    """Set multiple pixels to a range of colours sweeping from from_colour to to_colour.

    :param from_colour: A tuple with 3 values representing the red, green and blue of the first colour
    :param to_colour: A tuple with 3 values representing the red, green and blue of the second colour

    """
    if to_colour is None:
        to_colour = from_colour

    length = float(len(indexes))
    step = 0
    from_r, from_g, from_b = from_colour
    to_r, to_g, to_b = to_colour
    step_r, step_g, step_b = to_r - from_r, to_g - from_g, to_b - from_b
    step_r /= length
    step_g /= length
    step_b /= length

    for index in indexes:
        set_pixel(index, from_r + (step_r * step), from_g + (step_g * step), from_b + (step_b * step))
        step += 1



#!/usr/bin/env python

import time
from sys import exit

try:
    import psutil
except ImportError:
    exit('This script requires the psutil module\nInstall with: sudo pip install psutil')

import ledshim

ledshim.set_clear_on_exit()


def show_graph(v, r, g, b):
    v *= ledshim.NUM_PIXELS
    for x in range(ledshim.NUM_PIXELS):
        if v < 0:
            r, g, b = 0, 0, 0
        else:
            r, g, b = [int(min(v, 1.0) * c) for c in [r, g, b]]
        ledshim.set_pixel(x, r, g, b)
        v -= 1

    ledshim.show()


ledshim.set_brightness(0.6)

while True:
    v = psutil.cpu_percent() / 100.0
    show_graph(v, 255, 255, 255)
    time.sleep(0.01)



#!/usr/bin/env python

import colorsys
import math
import time

import ledshim

FALLOFF = 1.9
SCAN_SPEED = 4

ledshim.set_clear_on_exit()

start_time = time.time()

while True:
    delta = (time.time() - start_time)

    # Offset is a sine wave derived from the time delta
    # we use this to animate both the hue and larson scan
    # so they are kept in sync with each other
    offset = (math.sin(delta * SCAN_SPEED) + 1) / 2

    # Use offset to pick the right colour from the hue wheel
    hue = int(round(offset * 360))

    # Maximum number basex on NUM_PIXELS
    max_val = ledshim.NUM_PIXELS - 1

    # Now we generate a value from 0 to max_val
    offset = int(round(offset * max_val))

    for x in range(ledshim.NUM_PIXELS):
        sat = 1.0

        val = max_val - (abs(offset - x) * FALLOFF)
        val /= float(max_val)   # Convert to 0.0 to 1.0
        val = max(val, 0.0)     # Ditch negative values

        xhue = hue              # Grab hue for this pixel
        xhue += (1 - val) * 10  # Use the val offset to give a slight colour trail variation
        xhue %= 360             # Clamp to 0-359
        xhue /= 360.0           # Convert to 0.0 to 1.0

        r, g, b = [int(c * 255) for c in colorsys.hsv_to_rgb(xhue, sat, val)]

        ledshim.set_pixel(x, r, g, b, val / 4)

    ledshim.show()

    time.sleep(0.001)



#!/usr/bin/env python

import colorsys
import time
from sys import exit

try:
    import numpy as np
except ImportError:
    exit('This script requires the numpy module\nInstall with: sudo pip install numpy')

import ledshim

ledshim.clear()
start = 0
end = 60

while True:
    wait = np.random.choice(np.random.noncentral_chisquare(ledshim.NUM_PIXELS / 2, 1, 1000), 1)[0] / 50
    n = np.random.choice(np.random.noncentral_chisquare(ledshim.NUM_PIXELS / 2, 0.1, 1000), 1)
    limit = int(n[0])

    if limit > ledshim.NUM_PIXELS:
        limit = ledshim.NUM_PIXELS

    for pixel in range(limit):
        hue = start + (((end - start) / float(ledshim.NUM_PIXELS)) * pixel)
        r, g, b = [int(c * 255) for c in colorsys.hsv_to_rgb(hue / 360.0, 1.0, 1.0)]
        ledshim.set_pixel(pixel, r, g, b)
        ledshim.show()
        time.sleep(0.05 / (pixel + 1))

    time.sleep(wait)
    ledshim.clear()



#!/usr/bin/env python

import time
from sys import exit

try:
    from envirophat import motion
except ImportError:
    exit('This script requires the envirophat module\nInstall with: sudo pip install envirophat')

import blinkt

x_avg = []

while True:
    x, y, z = motion.accelerometer()

    x_avg.append(x)
    x_avg = x_avg[-5:]

    x = float(sum(x_avg)) / len(x_avg)
    x = max(-1, x)
    x = min(1, x)

    blinkt.clear()
    if abs(x) < 0.05:
        blinkt.set_pixel(3, 0, 255, 0)
        blinkt.set_pixel(4, 0, 255, 0)

    elif x > 0:
        val = abs(x) * 4.0
        for x in range(4):
            if val < 0:
                break
            blinkt.set_pixel(3 - x, int(255.0 * min(val, 1.0)), 0, 0)
            val -= 1

    elif x < 0:
        val = abs(x) * 4.0
        for x in range(4):
            if val < 0:
                break
            blinkt.set_pixel(4 + x, int(255.0 * min(val, 1.0)), 0, 0)
            val -= 1

    blinkt.show()

    time.sleep(0.01)



#!/usr/bin/env python

import glob
import os
import time
from sys import exit

try:
    import drumhat
except ImportError:
    exit('This script requires the drumhat module\nInstall with: sudo pip install drumhat')

try:
    import pygame
except ImportError:
    exit('This script requires the pygame module\nInstall with: sudo pip install pygame')

import blinkt

DRUM_FOLDER = 'drums2'

BANK = os.path.join(os.path.dirname(__file__), DRUM_FOLDER)

pygame.mixer.init(44100, -16, 1, 512)
pygame.mixer.set_num_channels(16)

files = glob.glob(os.path.join(BANK, '*.wav'))
files.sort()

samples = [pygame.mixer.Sound(f) for f in files]


def show_all(state):
    for i in range(8):
        val = state * 255
        blinkt.set_pixel(i, val, val, val)
    blinkt.show()


def handle_hit(event):
    samples[event.channel].play(loops=0)
    show_all(1)
    print('You hit pad {}, playing: {}'.format(event.pad, files[event.channel]))


def handle_release():
    show_all(0)


drumhat.on_hit(drumhat.PADS, handle_hit)
drumhat.on_release(drumhat.PADS, handle_release)

while True:
    time.sleep(1)



#!/usr/bin/env python

import random
import time

import ledshim

ledshim.set_clear_on_exit()
ledshim.set_brightness(0.4)

while True:
    for i in range(ledshim.NUM_PIXELS):
        ledshim.set_pixel(i, random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))

    ledshim.show()
    time.sleep(0.05)



#!/usr/bin/env python

import time

import ledshim

ledshim.set_clear_on_exit()
ledshim.set_brightness(0.5)


def show_all(state):
    for i in range(ledshim.NUM_PIXELS):
        val = state * 255
        ledshim.set_pixel(i, val, val, val)
    ledshim.show()


def dot():
    show_all(1)
    time.sleep(0.05)
    show_all(0)
    time.sleep(0.2)


def dash():
    show_all(1)
    time.sleep(0.2)
    show_all(0)
    time.sleep(0.2)


def space():
    time.sleep(0.2)


# 0 is a space, 1 is a dot and 2 is a dash
MORSE = '211101101211022101020120210212000'

while True:
    for m in MORSE:
        if m == '0':
            space()
        elif m == '1':
            dot()
        elif m == '2':
            dash()



#!/usr/bin/env python

import time

import ledshim

ledshim.set_clear_on_exit()

step = 0

while True:
    if step == 0:
        ledshim.set_all(128, 0, 0)

    if step == 1:
        ledshim.set_all(0, 128, 0)

    if step == 2:
        ledshim.set_all(0, 0, 128)

    step += 1
    step %= 3
    ledshim.show()
    time.sleep(0.5)



#!/usr/bin/env python

import time
from random import randint

import ledshim

MAX_SIZE = 4
MAX_GRID = ledshim.NUM_PIXELS + MAX_SIZE - 1

OFF = (0, 0, 0)
grid = [OFF] * (MAX_GRID + 1)

ledshim.set_clear_on_exit()


# The tetris algorithm fail when random_color was 0,0,0 now avoided
def random_color():
    return (randint(0, 255), randint(0, 255), randint(1, 50))


def random_tile(max_size, min_size=1):
    return (randint(min_size, max_size), random_color())


def place(tile):
    for i in range(0, tile[0]):
        grid[MAX_GRID - i - len(tile)] = tile[1]


def update():
    for i in range(ledshim.NUM_PIXELS):
        ledshim.set_pixel(i, grid[i][0], grid[i][1], grid[i][2])
    ledshim.show()


def has_lines():
    return grid[0] != OFF


def get_lines():
    lines = []
    for i, color in enumerate(grid):
        if color == OFF:
            return lines
        else:
            lines.append(i)
    return lines


def blink_lines():
    def hide():
        for line in get_lines():
            ledshim.set_pixel(line, 0, 0, 0)
        ledshim.show()

    hide()
    time.sleep(0.5)
    update()
    time.sleep(0.5)
    hide()
    time.sleep(0.5)


def remove_lines():
    for line in get_lines():
        grid[line] = OFF


def gravity():
    grid.append(OFF)
    grid.pop(0)


def main():
    ledshim.set_brightness(0.6)
    place(random_tile(MAX_SIZE))
    update()

    while True:
        time.sleep(0.5)

        if has_lines():
            blink_lines()
            remove_lines()
            place(random_tile(MAX_SIZE))
        else:
            gravity()

        update()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import time

import ledshim

# Just green
print('Green')
ledshim.set_multiple_pixels(range(0, 28), (0, 255, 0))
ledshim.show()
time.sleep(1)

# Sweep from yellow to blue, whee!
print('Yellow -> Blue')
ledshim.set_multiple_pixels(range(0, 28), from_colour=(255, 255, 0), to_colour=(0, 0, 255))
ledshim.show()
time.sleep(1)

# Half and half
print('Blue | Red -> Yellow')
ledshim.set_multiple_pixels(range(0, 14), from_colour=(0, 0, 255), to_colour=(128, 128, 255))
ledshim.set_multiple_pixels(range(14, 28), from_colour=(128, 0, 0), to_colour=(255, 255, 0))
ledshim.show()
time.sleep(1)



#!/usr/bin/env python

import colorsys
import math
import time

import ledshim

ledshim.set_clear_on_exit()

hue_range = 120
hue_start = 0
max_brightness = 0.8


def show_graph(v, r, g, b):
    v *= ledshim.NUM_PIXELS
    for x in range(ledshim.NUM_PIXELS):
        hue = ((hue_start + ((x / float(ledshim.NUM_PIXELS)) * hue_range)) % 360) / 360.0
        r, g, b = [int(c * 255) for c in colorsys.hsv_to_rgb(hue, 1.0, 1.0)]
        if v < 0:
            brightness = 0
        else:
            brightness = min(v, 1.0) * max_brightness

        ledshim.set_pixel(x, r, g, b, brightness)
        v -= 1

    ledshim.show()


ledshim.set_brightness(0.6)

try:
    while True:
        t = time.time() * 2
        v = (math.sin(t) + 1) / 2   # Get a value between 0 and 1
        show_graph(v, 255, 0, 255)  # Use it as the pixel index
        time.sleep(0.01)

except KeyboardInterrupt:
    pass



#!/usr/bin/env python

import time
from colorsys import hsv_to_rgb
from sys import exit

try:
    from envirophat import motion
except ImportError:
    exit('This script requires the envirophat module\nInstall with: sudo pip install envirophat')

import blinkt

offset = 0
direction = 0
t = []
t_start = 0

total_time = 1000


def millis():
    return int(round(time.time() * 1000))


while True:
    x, y, z = motion.accelerometer()
    print(y)

    if y < -1.9 and not direction == -1:
        direction = -1
        t.append(millis() - t_start)
        t_start = millis()
        t = t[-5:]

    if y > 1.9 and not direction == 1:
        direction = 1
        t.append(millis() - t_start)
        t_start = millis()
        t = t[-5:]

    if len(t) > 0:
        total_time = float(sum(t)) / len(t)

    offset = ((millis() - t_start) / total_time)

    # offset += direction * 10

    offset = min(1.0, offset)
    offset = max(0.0, offset)

    hue = offset

    if direction == -1:
        hue = 1.0 - offset

    r, g, b = [int(c * 255.0) for c in hsv_to_rgb(hue, 1.0, 1.0)]

    for x in range(8):
        blinkt.set_pixel(x, r, g, b)

    blinkt.show()

    time.sleep(0.0001)



#!/usr/bin/env python

import sys

import ledshim


def usage():
    print('Usage: {} <r> <g> <b>'.format(sys.argv[0]))
    sys.exit(1)


if len(sys.argv) != 4:
    usage()

# Exit if non integer value. int() will raise a ValueError
try:
    r, g, b = [int(x) for x in sys.argv[1:]]
except ValueError:
    usage()

# Exit if any of r, g, b are greater than 255
if max(r, g, b) > 255:
    usage()

print('Setting Blinkt to {r},{g},{b}'.format(r=r, g=g, b=b))

ledshim.set_clear_on_exit(False)

ledshim.set_all(r, g, b)

ledshim.show()



import time

import ledshim

for x in range(ledshim.width):
    ledshim.set_pixel(x,255,0,0)
    ledshim.show()
    time.sleep(0.05)

time.sleep(0.1)
ledshim.clear()
ledshim.show()

for x in range(ledshim.width):
    ledshim.set_pixel(x,0,255,0)
    ledshim.show()
    time.sleep(0.05)

time.sleep(0.1)
ledshim.clear()
ledshim.show()

for x in range(ledshim.width):
    ledshim.set_pixel(x,0,0,255)
    ledshim.show()
    time.sleep(0.05)



#!/usr/bin/env python

import math
import time

import ledshim

ledshim.set_clear_on_exit()


def show_graph(v, r, g, b):
    v *= ledshim.NUM_PIXELS
    for x in range(ledshim.NUM_PIXELS):
        if v < 0:
            r, g, b = 0, 0, 0
        else:
            r, g, b = [int(min(v, 1.0) * c) for c in [r, g, b]]
        ledshim.set_pixel(x, r, g, b)
        v -= 1

    ledshim.show()


ledshim.set_brightness(0.6)

try:
    while True:
        t = time.time()
        v = (math.sin(t) + 1) / 2   # Get a value between 0 and 1
        show_graph(v, 255, 0, 255)  # Use it as the graph intensity
        time.sleep(0.01)

except KeyboardInterrupt:
    pass



#!/usr/bin/env python

from sys import exit

try:
    import paho.mqtt.client as mqtt
except ImportError:
    exit('This example requires the paho-mqtt module\nInstall with: sudo pip install paho-mqtt')

import ledshim

MQTT_SERVER = 'iot.eclipse.org'
MQTT_PORT = 1883
MQTT_TOPIC = 'pimoroni/ledshim'

# Set these to use authorisation
MQTT_USER = None
MQTT_PASS = None

print("""
MQTT Blinkt! Control

This example uses public MQTT messages from {server} on port {port} to control Blinkt!

It will monitor the {topic} topic by default, and understands the following messages:

rgb,<pixel>,<r>,<g>,<b> - Set a single pixel to an RGB colour. Example: rgb,1,255,0,255
clr - Clear Blinkt!

You can use the online MQTT tester at http://www.hivemq.com/demos/websocket-client/ to send messages.

Use {server} as the host, and port 80 (Eclipse's websocket port). Set the topic to topic: {topic}
""".format(
    server=MQTT_SERVER,
    port=MQTT_PORT,
    topic=MQTT_TOPIC
))


def on_connect(client, userdata, flags, rc):
    print('Connected with result code ' + str(rc))

    client.subscribe(MQTT_TOPIC)


def on_message(client, userdata, msg):

    data = msg.payload.split(',')
    command = data.pop(0)

    if command == 'clr' and len(data) == 0:
        ledshim.clear()
        ledshim.show()
        return

    if command == 'rgb' and len(data) == 4:
        try:
            pixel = data.pop(0)

            if pixel == '*':
                pixel = None
            else:
                pixel = int(pixel)
                if pixel > 7:
                    print('Pixel out of range: ' + str(pixel))
                    return

            r, g, b = [int(x) & 0xff for x in data]

            print(command, pixel, r, g, b)

        except ValueError:
            print('Malformed command: ' + str(msg.payload))
            return

        if pixel is None:
            for x in range(ledshim.NUM_PIXELS):
                ledshim.set_pixel(x, r, g, b)
        else:
            ledshim.set_pixel(pixel, r, g, b)

        ledshim.show()
        return


ledshim.set_clear_on_exit()

client = mqtt.Client()
client.on_connect = on_connect
client.on_message = on_message

if MQTT_USER is not None and MQTT_PASS is not None:
    print('Using username: {un} and password: {pw}'.format(un=MQTT_USER, pw='*' * len(MQTT_PASS)))
    client.username_pw_set(username=MQTT_USER, password=MQTT_PASS)

client.connect(MQTT_SERVER, MQTT_PORT, 60)

client.loop_forever()



#!/usr/bin/env python

import time
from sys import exit

try:
    import requests
except ImportError:
    exit('This script requires the requests module\nInstall with: sudo pip install requests')

import ledshim

ledshim.set_clear_on_exit()


def hex_to_rgb(col_hex):
    """Convert a hex colour to an RGB tuple."""
    col_hex = col_hex.lstrip('#')
    return bytearray.fromhex(col_hex)


while True:
    r = requests.get('http://api.thingspeak.com/channels/1417/field/2/last.json', timeout=2)
    r, g, b = hex_to_rgb(r.json()['field2'])

    for i in range(ledshim.NUM_PIXELS):
        ledshim.set_pixel(i, r, g, b)

    ledshim.show()

    time.sleep(5)  # Be friendly to the API
